Exception: Pay::BraintreeAuthorizationError
- Inherits:
-
BraintreeError
- Object
- StandardError
- Error
- BraintreeError
- Pay::BraintreeAuthorizationError
- Defined in:
- lib/pay.rb
Instance Attribute Summary
Attributes inherited from BraintreeError
Instance Method Summary collapse
Methods inherited from BraintreeError
Constructor Details
This class inherits a constructor from Pay::BraintreeError
Instance Method Details
#message ⇒ Object
114 115 116 |
# File 'lib/pay.rb', line 114 def "Either the data you submitted is malformed and does not match the API or the API key you used may not be authorized to perform this action." end |