Class: Pay::PaddleClassic::Subscription
- Inherits:
-
Subscription
- Object
- Subscription
- Pay::PaddleClassic::Subscription
- Defined in:
- app/models/pay/paddle_classic/subscription.rb
Class Method Summary collapse
Instance Method Summary collapse
- #api_record(**options) ⇒ Object
-
#cancel(**options) ⇒ Object
Paddle subscriptions are canceled immediately, however we still want to give the user access to the end of the period they paid for.
- #cancel_now!(**options) ⇒ Object
- #change_quantity(quantity, **options) ⇒ Object
-
#on_grace_period? ⇒ Boolean
A subscription could be set to cancel or pause in the future It is considered on grace period until the cancel or pause time begins.
- #pause ⇒ Object
- #paused? ⇒ Boolean
- #resumable? ⇒ Boolean
- #resume ⇒ Object
-
#retry_failed_payment ⇒ Object
Retries the latest invoice for a Past Due subscription.
- #swap(plan, **options) ⇒ Object
Class Method Details
.sync(subscription_id, object: nil, name: Pay.default_product_name) ⇒ Object
7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 |
# File 'app/models/pay/paddle_classic/subscription.rb', line 7 def self.sync(subscription_id, object: nil, name: Pay.default_product_name) # Passthrough is not return from this API, so we can't use that object ||= PaddleClassic.client.users.list(subscription_id: subscription_id).data.try(:first) pay_customer = Pay::Customer.find_by(processor: :paddle_classic, processor_id: object.user_id) # If passthrough exists (only on webhooks) we can use it to create the Pay::Customer if pay_customer.nil? && object.passthrough owner = Pay::PaddleClassic.owner_from_passthrough(object.passthrough) pay_customer = owner&.set_payment_processor(:paddle_classic, processor_id: object.user_id) end return unless pay_customer attributes = { paddle_cancel_url: object.cancel_url, paddle_update_url: object.update_url, processor_plan: object.plan_id || object.subscription_plan_id, quantity: object.quantity || 1, status: object.state || object.status } case attributes[:status] when "trialing" attributes[:trial_ends_at] = Time.zone.parse(object.next_bill_date) attributes[:ends_at] = nil when "active", "past_due" attributes[:trial_ends_at] = nil attributes[:ends_at] = nil when "paused", "deleted" # If paused or delete while on trial, set ends_at to match attributes[:trial_ends_at] = nil attributes[:ends_at] = Time.zone.parse(object.next_bill_date) end # Update or create the subscription if (pay_subscription = pay_customer.subscriptions.find_by(processor_id: object.subscription_id)) pay_subscription.with_lock do pay_subscription.update!(attributes) end pay_subscription else pay_customer.subscriptions.create!(attributes.merge(name: name, processor_id: object.subscription_id)) end end |
Instance Method Details
#api_record(**options) ⇒ Object
53 54 55 56 57 |
# File 'app/models/pay/paddle_classic/subscription.rb', line 53 def api_record(**) PaddleClassic.client.users.list(subscription_id: processor_id).data.try(:first) rescue ::Paddle::Error => e raise Pay::PaddleClassic::Error, e end |
#cancel(**options) ⇒ Object
Paddle subscriptions are canceled immediately, however we still want to give the user access to the end of the period they paid for
60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 |
# File 'app/models/pay/paddle_classic/subscription.rb', line 60 def cancel(**) return if canceled? ends_at = if on_trial? trial_ends_at elsif paused? pause_starts_at else Time.parse(api_record.next_payment.date) end PaddleClassic.client.users.cancel(subscription_id: processor_id) update( status: (ends_at.future? ? :active : :canceled), ends_at: ends_at ) # Remove payment methods since customer cannot be reused after cancelling Pay::PaymentMethod.where(customer_id: customer_id).destroy_all rescue ::Paddle::Error => e raise Pay::PaddleClassic::Error, e end |
#cancel_now!(**options) ⇒ Object
83 84 85 86 87 88 89 90 91 92 93 |
# File 'app/models/pay/paddle_classic/subscription.rb', line 83 def cancel_now!(**) return if canceled? PaddleClassic.client.users.cancel(subscription_id: processor_id) update(status: :canceled, ends_at: Time.current) # Remove payment methods since customer cannot be reused after cancelling Pay::PaymentMethod.where(customer_id: customer_id).destroy_all rescue ::Paddle::Error => e raise Pay::PaddleClassic::Error, e end |
#change_quantity(quantity, **options) ⇒ Object
95 96 97 |
# File 'app/models/pay/paddle_classic/subscription.rb', line 95 def change_quantity(quantity, **) raise NotImplementedError, "Paddle does not support setting quantity on subscriptions" end |
#on_grace_period? ⇒ Boolean
A subscription could be set to cancel or pause in the future It is considered on grace period until the cancel or pause time begins
101 102 103 |
# File 'app/models/pay/paddle_classic/subscription.rb', line 101 def on_grace_period? (canceled? && Time.current < ends_at) || (paused? && pause_starts_at? && Time.current < pause_starts_at) end |
#pause ⇒ Object
109 110 111 112 113 114 |
# File 'app/models/pay/paddle_classic/subscription.rb', line 109 def pause response = PaddleClassic.client.users.pause(subscription_id: processor_id) update(status: :paused, pause_starts_at: Time.zone.parse(response.dig(:next_payment, :date))) rescue ::Paddle::Error => e raise Pay::PaddleClassic::Error, e end |
#paused? ⇒ Boolean
105 106 107 |
# File 'app/models/pay/paddle_classic/subscription.rb', line 105 def paused? status == "paused" end |
#resumable? ⇒ Boolean
116 117 118 |
# File 'app/models/pay/paddle_classic/subscription.rb', line 116 def resumable? paused? end |
#resume ⇒ Object
120 121 122 123 124 125 126 127 128 129 |
# File 'app/models/pay/paddle_classic/subscription.rb', line 120 def resume unless resumable? raise Error, "You can only resume paused subscriptions." end PaddleClassic.client.users.unpause(subscription_id: processor_id) update(ends_at: nil, status: :active, pause_starts_at: nil) rescue ::Paddle::Error => e raise Pay::PaddleClassic::Error, e end |
#retry_failed_payment ⇒ Object
Retries the latest invoice for a Past Due subscription
144 145 |
# File 'app/models/pay/paddle_classic/subscription.rb', line 144 def retry_failed_payment end |
#swap(plan, **options) ⇒ Object
131 132 133 134 135 136 137 138 139 140 141 |
# File 'app/models/pay/paddle_classic/subscription.rb', line 131 def swap(plan, **) raise ArgumentError, "plan must be a string" unless plan.is_a?(String) attributes = {plan_id: plan, prorate: .fetch(:prorate) { true }} attributes[:quantity] = quantity if quantity? PaddleClassic.client.users.update(subscription_id: processor_id, **attributes) update(processor_plan: plan, ends_at: nil, status: :active) rescue ::Paddle::Error => e raise Pay::PaddleClassic::Error, e end |