Class: Pay::Stripe::Webhooks::SubscriptionCreated
- Inherits:
-
SubscriptionUpdated
- Object
- SubscriptionUpdated
- Pay::Stripe::Webhooks::SubscriptionCreated
- Defined in:
- lib/pay/stripe/webhooks/subscription_created.rb
Overview
If a subscription is manually created on Stripe, we want to sync it the same way as an update